#include <stdio.h>
#define _USE_MATH_DEFINES
#include "Network.h"

#include <BWAPI.h>
#include <BWAPI/Client.h>
#include <BWTA.h>
#include <iostream>
#include <fstream>
#include <list>
#include <vector>
#include <string>
#include <sstream>
#include<math.h>

#include <windows.h>

#include <string>

#include "Population.h"

using namespace BWAPI;

void drawStats();
void drawBullets();
void drawVisibilityData();
void drawTerrainData();
void showPlayers();
void showForces();
bool show_bullets;
bool show_visibility_data;

bool analyzed;
bool analysis_just_finished;
BWTA::Region* home;
BWTA::Region* enemy_base;
DWORD WINAPI AnalyzeThread();
Unit* center;
UnitType type;
int trycount = 0;

void reconnect()
{
  while(!BWAPIClient.connect())
  {
    Sleep(1000);
  }
}
std::vector<double> relativePosition(Position origin, Position relative, int sightrange)
{
	std::vector<double> retur;

	double diffX = relative.x()-origin.x();
	double diffY = relative.y()-origin.y();

	double relX;
	double relY;

	if(abs(diffX) > sightrange)
	{
		if(diffX > 0)
		{
			relX = 1;
		}
		else
		{
			relX = -1;
		}
	}
	else
	{
		relX = diffX/sightrange;
	}
	
	if(abs(diffY) > 7)
	{
		if(diffY > 0)
		{
			relY = 1;
		}
		else
		{
			relY = -1;
		}
	}
	else
	{
		relY = diffY/sightrange;
	}

	double X = (relX+1)/2;
	double Y = (relY+1)/2;

	retur.push_back(X);
	retur.push_back(Y);

	return retur;
}
bool closeComparison(Unit* a, Unit* b)
{
	return center->getDistance(a) < center->getDistance(b);
}
void situationTracker(string filename, int sightrange)
{
	  std::ofstream file(filename.c_str(), std::ios::app);
	  vector<string> situations;
	  
	  for(std::set<Unit*>::const_iterator i=Broodwar->self()->getUnits().begin();i!=Broodwar->self()->getUnits().end();i++)
	  {
		  if(!((*i)->isSelected()))
		  {
			  continue;
		  }
#pragma region ownstate
		  std::stringstream action;
		  std::stringstream situation;
		  int x = 0;
		  int y = 0;
		  double angle = 0.0;
		  int hitpoints = 0;
		  int mapWidth = Broodwar->mapWidth();
		  int mapHeight = Broodwar->mapHeight();
		  UnitType type = (*i)->getType();
		  WeaponType weapon = type.groundWeapon();
		  center = (*i);

		  
		  //Agent position 
		  Position own = (*i)->getPosition();
		  //situation << own.x() << " " << own.y() << " ";
		  //Agent heading
		  double heading = (*i)->getAngle();
		  situation << heading/(2*(M_PI)) << " ";
		  //Agent hit points
		  int ownHP = (*i)->getHitPoints();
		  situation << ownHP/type.maxHitPoints() << " ";
		  //Agent weapon status
		  int weaponCooldown = ((*i)->getGroundWeaponCooldown());
		  situation << weaponCooldown/weapon.damageCooldown() << " ";
		  //Agent underattack
		  bool underAttack = (*i)->isUnderAttack();
		  situation << underAttack << " ";

		  std::list<Unit*> allies;
		  std::list<Unit*> enemies;
		

		  for(std::set<Unit*>::const_iterator j=Broodwar->self()->getUnits().begin();j!=Broodwar->self()->getUnits().end();j++)
		  {
			  allies.push_back((*j));
			  x += (*j)->getPosition().x();
			  y += (*j)->getPosition().y();
			  angle += (*j)->getAngle();
			  hitpoints += (*j)->getHitPoints();
		  }
		  
		  if(allies.size() > 0)
		  {
			  x /= allies.size();
			  y /= allies.size();
			  angle /= allies.size(); 
		  }
		  //Centroid position of allies
		  Position alliesCentroid(x, y);
		  vector<double> aCentroid = relativePosition(own, alliesCentroid, sightrange);
		  situation << aCentroid[0] << " " << aCentroid[1] << " ";
		  //Centroid angle of allies 
		  double alliesCentrAngle = angle;
		  situation << alliesCentrAngle/(2*(M_PI)) << " ";
		  //Total Hitpoints of allies
		  int alliesHP = hitpoints;
		  //situation << alliesHP << " ";
		  //sorting list of allies by position to current unit
		  allies.sort(closeComparison);
		  //Popping the first element to get rid of the original unit
		  allies.pop_front();
		  //position of the three closest allies, replaced with -1 if fewer units than 3 are left
		  for(int h = 0; h < 3; h++)
		  {
			  if(allies.size() > 0)
			  {  
				  vector<double> relpos = relativePosition(own, allies.front()->getPosition(), sightrange);
			  
				  situation << relpos[0] << " " << relpos[1] << " ";

				  allies.pop_front();
			  }
			  else
			  {
				  situation << aCentroid[0] << " " << aCentroid[1] << " ";
			  }
		  }
#pragma endregion
		  //resetting variables
		  x = 0;
		  y = 0;
		  angle = 0.0;
		  hitpoints = 0;
		  
		  for(std::set<Unit*>::const_iterator j = Broodwar->enemy()->getUnits().begin(); j!=Broodwar->enemy()->getUnits().end(); j++)
		  {
			  enemies.push_back((*j));
			  x += (*j)->getPosition().x();
			  y += (*j)->getPosition().y();
			  angle += (*j)->getAngle();
			  hitpoints += (*j)->getHitPoints();
		  }
			
		  if(enemies.size() > 0)
		  {
			  x /= enemies.size();
			  y /= enemies.size();
			  angle /= enemies.size();
		  }
		  //centroid of enemies
		  Position enemiesCentroid(x, y);
		  vector<double> eCentroid = relativePosition(own, enemiesCentroid, sightrange);
		  situation << eCentroid[0] << " " << eCentroid[1] << " ";
		  //Centroid angle of enemies
		  double enemyCentrAngle = angle;
		  situation << enemyCentrAngle/2*(M_PI) << " ";
		  //total enemy hitpoints
		  int enemyHP = hitpoints;
		  double hpRatio = 0;
		  if(enemyHP != 0)
		  {
			  hpRatio = alliesHP/enemyHP;
		  }
		  situation << hpRatio << " ";
		  //sort enemies by position to current unit
		  enemies.sort(closeComparison);
		  //vector containing the three closest enemies
		  vector<Unit*> threeEnemies;

		  for(int x = 0; x < 3; x++)
		  {
			  if(enemies.size() > 0)
			  {
				  //position of the closest enemies up to three
				  Position pos = enemies.front()->getPosition();
				  vector<double> relpos = relativePosition(own, pos, sightrange);
				  situation << relpos[0] << " " << relpos[1] << " ";
				  //hp of closest enemies
				  int eHP = enemies.front()->getHitPoints();
				  situation << eHP/enemies.front()->getType().maxHitPoints() << " ";
				  //Wether or not this enemy was recently attacked
				  bool eUnderAttack = enemies.front()->isUnderAttack();
				  situation << eUnderAttack << " ";
				  //pushin the unit* to vector and removing it from the list
				  threeEnemies.push_back(enemies.front());
				  enemies.pop_front();
			  }
			  else
			  {
				  //if fewer than three enemies left print xy coordinates -1 -1
				  situation << eCentroid[0] << " ";
				  situation << eCentroid[1] << " ";
				  //hitpoints and underattack both 0
				  situation << 0 << " " << 0 << " ";
			  }
		  }
#pragma region needs fixing
		  /*
		  std::set<Unit*> inRange = (*i)->getUnitsInRadius(7);
		  //push them into a list
		  std::list<Unit*> closeUnits;
		  for(std::set<Unit*>::const_iterator j = inRange.begin(); j != inRange.end(); j++)
		  {
			  closeUnits.push_back((*j));
		  }
		  //sort the list by closeness to current unit
		  closeUnits.sort(closeComparison);
		  //variables for local strength ratios
		  //hp ratios
		  double closeAllyHP = 0.0;
		  double closeEnemyHP = 0.0;
		  //number of units ratios
		  int numCloseAllies = 0;
		  int numCloseEnemies = 0;
		  //highground ratios
		  int numAlliesOnHG = 0;
		  int numEnemiesOnHG = 0;
		  
		  for(std::list<Unit*>::iterator j = closeUnits.begin(); j != closeUnits.end(); j++)
		  {
			  //if the unit is owned by the player
			  if((*j)->getPlayer() == Broodwar->self())
			  {
				  closeAllyHP += (*j)->getHitPoints();
				  numCloseAllies++;
				  if((*j)->getRegion()->isHigherGround())
				  {
					  numAlliesOnHG++;
				  }
			  }
			  //if the unit is owned by the enemy
			  else if((*j)->getPlayer() == Broodwar->enemy())
			  {
				  closeEnemyHP += (*j)->getHitPoints();
				  numCloseEnemies++;
				  if((*j)->getRegion()->isHigherGround())
				  {
					  numEnemiesOnHG++;
				  }
			  }
		  }
		  double aEHpRatio = 0;
		  double aENumRatio = 0;
		  double aEHgRatio = 0;
		  if(closeEnemyHP > 0)
		  {
			  aEHpRatio = closeAllyHP/closeEnemyHP;
		  }
		  if(numCloseEnemies > 0)
		  {
			  aENumRatio = numCloseAllies/numCloseEnemies;
		  }
		  if(numEnemiesOnHG > 0)
		  {
			  aEHgRatio = numAlliesOnHG/numEnemiesOnHG;
		  }
		  situation << aEHpRatio << " " << aENumRatio << " " << aEHgRatio << " ";
		  */
#pragma endregion
#pragma region actiontaken	  
		  //recording the action taken
		  //check to see if the unit is currently moving
		  if((*i)->isMoving())
		  {
			  //get target position i.e the position the unit has been told to go to
			  Position posi = (*i)->getTargetPosition();
			  vector<double> relpos = relativePosition(own, posi, sightrange);
			  //write the action i.e moving to x,y 0 0 0 to attack
			  action << (*i)->isMoving() << " " << relpos[0] << " " << relpos[1] << " " << 0 << " " << 0 << " " << 0;
		  }
		  //check to see if the unit has been ordered to attack
		  else if((*i)->getOrder().getName() == "AttackUnit")
		  {	
			  //id of target
			  int target;
			  //check to see if the unit currently has a target
			  if((*i)->getOrderTarget() != NULL)
			  {
				  //set target to the id of the currently targeted unit
				  target = (*i)->getOrderTarget()->getID();
			  }
			  else if((*i)->getTarget() != NULL)
			  {
				  target = (*i)->getTarget()->getID();
			  }
			  else
			  {
				  //if no target exists move on to next iteration
				  continue;
			  }
			 
			  //write that the unit is attacking and and that the move to cordinates are -1 -1
			  action << (*i)->isMoving() << " " << 0 << " " << 0;
			  //iterate through the list of three closest enemies checking the target id against the id of the closest enemies
			  bool hasTarget = false;
			  for(unsigned int f = 0; f < 3; f++)
			  {
				  if(target != NULL && threeEnemies.size() > f)
				  {   
					  printf("target: %i actual: %i \n", target, threeEnemies[f]->getID());
					  if(target == threeEnemies[f]->getID())
					  {
						  //print 1 if the unit is the one currently attacked
						  action << " " << 1;
						  hasTarget = true;
					  }
					  else
					  {
						  //if not print 0
						  action << " " << 0;
					  }
				  }
				  else if(target != NULL)
				  {
					  //if threenemies has fewer than three elements print 0 for the missing members
					  action << " " << 0;
				  }
				  else if(!hasTarget)
				  {
					  continue;
				  }
				  else
				  {
					  printf("Order target = NULL \n");
					  continue;
				  }
			  }
		  }
		  //if an action has been recorded push the strings to vector
		  //if not take no action
		  if(action.str().length() > 0 && action.str() != "0 0 0 0 0 0")
		  {
			situations.push_back(situation.str());
			situations.push_back(action.str());
		  }
#pragma endregion
	}
	for(unsigned int h = 0; h < situations.size(); h++)
	{
		file << situations[h];
		file << "\n";
	}
	file.close();
}
std::vector< std::vector<double> > readFromFile(string filename)
{
	std::vector< std::vector<double> > results;
	std::ifstream file(filename.c_str());
	std::stringstream input;
	input << file.rdbuf();
	file.close();

	std::string inputStr;

	std::stringstream file_inputs;
	std::stringstream file_outputs;
	int evenCheck = 0;
	int numberOfExAmples = 0;
	while(std::getline(input, inputStr))
	{
		if(evenCheck%2 == 0 && inputStr.size() != 0)
		{
			file_inputs << inputStr << " \n";
			numberOfExAmples++;
		}
		else if(inputStr.size() != 0)
		{
			file_outputs << inputStr << " \n";
		}
		else
		{
			continue;
		}
		evenCheck++;
	}
	vector<double> inFromFile;
	vector<double> outFromFile;
	vector<double> dimensions;

	double temp;

	while(file_inputs >> temp)
	{
		inFromFile.push_back(temp);
	}
	while(file_outputs >> temp)
	{	
		outFromFile.push_back(temp);
	}

	dimensions.push_back(numberOfExAmples);
	dimensions.push_back(inFromFile.size()/numberOfExAmples);
	dimensions.push_back(outFromFile.size()/numberOfExAmples);

	results.push_back(inFromFile);
	results.push_back(outFromFile);
	results.push_back(dimensions);
	return results;
}
void WriteToFile(vector< vector<double> > vec, string filename)
{
	std::ofstream file(filename.c_str());
	for(unsigned int i = 0; i < vec.size(); i++)
	{
		for(unsigned int j = 0; j < vec[i].size(); j++)
		{
			file << vec[i][j] << " ";
		}
	}
	file << "\n";
	file.close();
}
void writeBestToFile(vector<double> best, string filename, double bestfitness, double avg, int gen)
{
	std::ofstream file(filename.c_str(), std::ios::app);
	file << "gen: " << gen << " bestfitness: " << bestfitness << " avg: " << avg << "\n";
	for(unsigned int i = 0; i < best.size(); i++)
	{
		file << best[i] << " ";
	}
	file << "\n \n";
	file.close();

}
vector<double> readWeightsFromFile(string filename)
{
	std::ifstream file(filename.c_str());
	std::stringstream weightFromFIle;

	vector<double> weights;
	weightFromFIle << file.rdbuf();
	file.close();
	double temp;
	while(weightFromFIle >> temp)
	{
		weights.push_back(temp);
	}

	return weights;
}
Network createNetwork(string filename, int* layers, int numlayers)
{
	vector< vector<double> > input = readFromFile(filename);

	layers[0] = input[2][1];
	layers[numlayers-1] = input[2][2];

	Network neuralNet(numlayers, layers);

	neuralNet.backProp(input[0], input[1], input[2][0], 0.02, 0.1);

	return neuralNet;
}
Network loadNetwork(string filename, int* layers, int numlayers)
{
	
	vector<double> weights = readWeightsFromFile(filename);
	
	Network ann(numlayers, layers);

	ann.changeWeights(weights);

	return ann;
}
vector<double> getSituation(Unit* i, int sightrange)
{
	std::stringstream situation;
	int x = 0;
	int y = 0;
	double angle = 0.0;
	int hitpoints = 0;
	int mapWidth = Broodwar->mapWidth();
	int mapHeight = Broodwar->mapHeight();
	UnitType type = i->getType();
	WeaponType weapon = type.groundWeapon();
	center = i;


	//Agent position 
	Position own = i->getPosition();
	//situation << own.x() << " " << own.y() << " ";
	//Agent heading
	double heading = i->getAngle();
	situation << heading/(2*(M_PI)) << " ";
	//Agent hit points
	int ownHP = i->getHitPoints();
	situation << ownHP/type.maxHitPoints() << " ";
	//Agent weapon status
	int weaponCooldown = i->getGroundWeaponCooldown();
	situation << weaponCooldown/weapon.damageCooldown() << " ";
	//Agent underattack
	bool underAttack = i->isUnderAttack();
	situation << underAttack << " ";

	std::list<Unit*> allies;
	std::list<Unit*> enemies;


	for(std::set<Unit*>::const_iterator j=Broodwar->self()->getUnits().begin();j!=Broodwar->self()->getUnits().end();j++)
	{
	  allies.push_back((*j));
	  x += (*j)->getPosition().x();
	  y += (*j)->getPosition().y();
	  angle += (*j)->getAngle();
	  hitpoints += (*j)->getHitPoints();
	}

	if(allies.size() > 0)
	{
	  x /= allies.size();
	  y /= allies.size();
	  angle /= allies.size(); 
	}
	//Centroid position of allies
	Position alliesCentroid(x, y);
	vector<double> aCentroid = relativePosition(own, alliesCentroid, sightrange);
	situation << aCentroid[0] << " " << aCentroid[1] << " ";
	//Centroid angle of allies 
	double alliesCentrAngle = angle;
	situation << alliesCentrAngle/(2*(M_PI)) << " ";
	//Total Hitpoints of allies
	int alliesHP = hitpoints;
	//situation << alliesHP << " ";
	//sorting list of allies by position to current unit
	allies.sort(closeComparison);
	//Popping the first element to get rid of the original unit
	allies.pop_front();
	//position of the three closest allies, replaced with -1 if fewer units than 3 are left
	for(int h = 0; h < 3; h++)
	{
	  if(allies.size() > 0)
	  {  
		  vector<double> relpos = relativePosition(own, allies.front()->getPosition(), sightrange);
	  
		  situation << relpos[0] << " " << relpos[1] << " ";

		  allies.pop_front();
	  }
	  else
	  {
		  situation << aCentroid[0] << " " << aCentroid[1] << " ";
	  }
	}
	#pragma endregion
	//resetting variables
	x = 0;
	y = 0;
	angle = 0.0;
	hitpoints = 0;

	for(std::set<Unit*>::const_iterator j = Broodwar->enemy()->getUnits().begin(); j!=Broodwar->enemy()->getUnits().end(); j++)
	{
	  enemies.push_back((*j));
	  x += (*j)->getPosition().x();
	  y += (*j)->getPosition().y();
	  angle += (*j)->getAngle();
	  hitpoints += (*j)->getHitPoints();
	}

	if(enemies.size() > 0)
	{
	  x /= enemies.size();
	  y /= enemies.size();
	  angle /= enemies.size();
	}
	//centroid of enemies
	Position enemiesCentroid(x, y);
	vector<double> eCentroid = relativePosition(own, enemiesCentroid, sightrange);
	situation << eCentroid[0] << " " << eCentroid[1] << " ";
	//Centroid angle of enemies
	double enemyCentrAngle = angle;
	situation << enemyCentrAngle/2*(M_PI) << " ";
	//total enemy hitpoints
	int enemyHP = hitpoints;
	double hpRatio = 0;
	if(enemyHP != 0)
	{
	  hpRatio = alliesHP/enemyHP;
	}
	situation << hpRatio << " ";
	//sort enemies by position to current unit
	enemies.sort(closeComparison);
	//vector containing the three closest enemies
	vector<Unit*> threeEnemies;

	for(int x = 0; x < 3; x++)
	{
	  if(enemies.size() > 0)
	  {
		  //position of the closest enemies up to three
		  Position pos = enemies.front()->getPosition();
		  vector<double> relpos = relativePosition(own, pos, sightrange);
		  situation << relpos[0] << " " << relpos[1] << " ";
		  //hp of closest enemies
		  int eHP = enemies.front()->getHitPoints();
		  situation << eHP/enemies.front()->getType().maxHitPoints() << " ";
		  //Wether or not this enemy was recently attacked
		  bool eUnderAttack = enemies.front()->isUnderAttack();
		  situation << eUnderAttack << " ";
		  //pushin the unit* to vector and removing it from the list
		  threeEnemies.push_back(enemies.front());
		  enemies.pop_front();
	  }
	  else
	  {
		  //if fewer than three enemies left print xy coordinates of centroid
		  situation << eCentroid[0] << " ";
		  situation << eCentroid[1] << " ";
		  //hitpoints and underattack both 0
		  situation << 0 << " " << 0 << " ";
	  }
	}
	vector<double> sit;
	double temp;
	while(situation >> temp)
	{
		sit.push_back(temp);
	}
	return sit;
}
vector<double> computeAction(vector<double> input, Network ANN)
{
	ANN.activate(input, input.size());
	return ANN.getResults();
}
vector<double> getWeightsSingleVector(Network ann)
{
	vector< vector<double> > v = ann.getWeights();
	vector<double> singleVector = v[0];

	for(unsigned int i = 1; i < v.size(); i++)
	{
		for(unsigned int j = 0; j < v[i].size(); j++)
		{	
			singleVector.push_back(v[i][j]);
		}
	}
	return singleVector;
}
void giveOrders(Network ANN, int sightrange)
{
	for(set<Unit*>::const_iterator i = Broodwar->self()->getUnits().begin(); i != Broodwar->self()->getUnits().end(); i++)
	{
		if((*i)->isSelected())
		{
			continue;
		}

		type = (*i)->getType();

		vector<double> situation = getSituation((*i), sightrange);
		vector<double> action = computeAction(situation, ANN);
		center = (*i);
		if(action[0] > 0.8 && !(*i)->isAttacking() && !((*i)->isAttackFrame()))
		{
			
			Position own = (*i)->getPosition();
			int xdiff = ((2*action[1])-1)*sightrange;
			int ydiff = ((2*action[2])-1)*sightrange;
			Position p(own.x()+xdiff, own.y()+ydiff);
			(*i)->move(p);
		}
		else if(action[0] < 0.2)
		{
			
			std::set<Unit*> enemies = Broodwar->enemy()->getUnits();
			std::list<Unit*> enemiesList(enemies.begin(), enemies.end());
			enemiesList.sort(closeComparison);
			if(action[3] > 0.8 && enemiesList.size() > 0)
			{
				(*i)->attack(enemiesList.front());
			}
			else if(action[4] > 0.8 && enemiesList.size() > 1)
			{
				enemiesList.pop_front();
				(*i)->attack(enemiesList.front());
			}
			else if(action[5] > 0.8 && enemiesList.size() > 2)
			{
				enemiesList.pop_front();
				enemiesList.pop_front();
				(*i)->attack(enemiesList.front());
			}
			else if(enemiesList.size() > 0)
			{
				(*i)->attack(enemiesList.front());
			}
		}
	}
}

int main(int argc, const char* argv[])
{
	srand((unsigned)time(0));

	BWAPI::BWAPI_init();
	
	UnitType type;
	int counter = 0;
	int wins = 0;
	int losses = 0;
	double score = 0;
	double dead = 0;

	int numIndividuals = 20;
	int numgenerations = 100;

	int gencount = 0;
	int indcount = 0;
	int trialscount = 0;
	int trialsPerInd = 5;

	bool elitism = true;

	double mutationprob = 0.5;
	double deviation = 1;
	double mean = 0.0;
	double percentage = 0.3;

	bool shouldChange = true;

	bool testmode = true;
	int trynumber = 15;
	string pathString = "backprop\\5seeds10mut051elite03.txt";//"25pop0305mutation100gen20trials\\eliteEvo20trials.txt";

	vector< vector<double> > seeds;
	
	vector<double> fitnesses;

	int framemodulo = 15; //how often the give orders method is run 1 = evry frame, 5 = evry 5 frames

	int layers[4] = {29, 5, 6};
	
	//vector< vector<double> > bpexamples = readFromFile("test1.txt");

	Network ann = loadNetwork("backprop\\evoresults\\test.txt", layers, 3);// 19try14V221score createNetwork("test1.txt", layers, 3) last full 20 gen "test1\\5h5h3v152score.txt"; 900scoregen22.txt seed for final.txt seed for next tests
	
	//ann.backProp(bpexamples[0], bpexamples[1], bpexamples[2][0], 0.02, 0.1);

	//Network ann = createNetwork("test1.txt", layers, 3);
	/*
	
	vector<double> weights00 = readWeightsFromFile("900scoregen22.txt");
	vector<double> weights0 = readWeightsFromFile("final.txt");
	vector<double> weights1 = readWeightsFromFile("test1\\5h5h3v152score.txt");
	vector<double> weights2 = readWeightsFromFile("test1\\5h3v137score.txt");
	vector<double> weights3 = readWeightsFromFile("test1\\5h5h0v121score.txt");
	vector<double> weights4 = readWeightsFromFile("test2\\5h6v164score.txt");
	vector<double> weights5 = readWeightsFromFile("test2\\5hn1V71score.txt");

	*/

	//vector<double> weights00 = readWeightsFromFile("25pop0305mutation100gen20trials\\5h59V395score.txt");
	vector<double> weights0 = readWeightsFromFile("backprop\\20try19V250score.txt");
	vector<double> weights1 = readWeightsFromFile("backprop\\17try17V225score.txt");
	vector<double> weights2 = readWeightsFromFile("backprop\\1try16V249score.txt");
	vector<double> weights3 = readWeightsFromFile("backprop\\14try14V243score.txt");
	vector<double> weights4 = readWeightsFromFile("backprop\\19try14V221score.txt");
	//vector<double> weights5 = readWeightsFromFile("backprop\\15try13V163score.txt");
	//vector<double> weights6 = readWeightsFromFile("25pop0305mutation100gen20trials\\5h79V453score.txt");
	//vector<double> weights7 = readWeightsFromFile("25pop0305mutation100gen20trials\\5h82V461score.txt");

	//seeds.push_back(weights00);
	seeds.push_back(weights0);
	seeds.push_back(weights1);
	seeds.push_back(weights2);
	seeds.push_back(weights3);
	seeds.push_back(weights4);
	//seeds.push_back(weights5);
	//seeds.push_back(weights6);
	//seeds.push_back(weights7);

	vector<double> seed = getWeightsSingleVector(ann);
	//Population nr1(numIndividuals, 175, 0, 1.5);//(numIndividuals, seeds);
	Population nr1(numIndividuals, seeds);
	

	printf("Connecting...");
	reconnect();
	while(true)
	{
		//printf("waiting to enter match\n");
		while (!Broodwar->isInGame())
		{
			BWAPI::BWAPIClient.update();
			if (!BWAPI::BWAPIClient.isConnected())
			{
				printf("Reconnecting...\n");
				reconnect();
			}
		}
	
		int maxwidth = Broodwar->mapWidth() * 32;
		int maxheight = Broodwar->mapHeight() * 32;



		//printf("Going to: %i x %i y \n", x, y);

		//printf("starting match!");
		//Broodwar->printf("The map is %s, a %d player",Broodwar->mapName().c_str(),Broodwar->getStartLocations().size());
		// Enable some cheat flags
		Broodwar->enableFlag(Flag::UserInput);
		// Uncomment to enable complete map information
		Broodwar->enableFlag(Flag::CompleteMapInformation);

		//read map information into BWTA so terrain analysis can be done in another thread
		BWTA::readMap();
		analyzed=false;
		analysis_just_finished=false;

		show_bullets=false;
		show_visibility_data=false;
	
		if (Broodwar->isReplay())
		{
			Broodwar->printf("The following players are in this replay:");
			for(std::set<Player*>::iterator p=Broodwar->getPlayers().begin();p!=Broodwar->getPlayers().end();p++)
		{
        if (!(*p)->getUnits().empty() && !(*p)->isNeutral())
        {
          Broodwar->printf("%s, playing as a %s",(*p)->getName().c_str(),(*p)->getRace().getName().c_str());
        }
      }
    }
    else
    {
      Broodwar->printf("The match up is %s v %s",
        Broodwar->self()->getRace().getName().c_str(),
        Broodwar->enemy()->getRace().getName().c_str());

    }
	
	Broodwar->setLocalSpeed(0);
	//Broodwar->setFrameSkip(100);
    while(Broodwar->isInGame())
    {
		int frame = Broodwar->getFrameCount();
		if(!testmode && shouldChange && indcount < numIndividuals && trialscount < trialsPerInd)
		{
			chromosome subject = nr1.getIndividual(indcount);
			ann.changeWeights(subject.getGenes());
			shouldChange = false;
			printf("changing subject Current subject id: %i gen: %i \n", subject.getID(), subject.gen);
		}
		
		if(frame%framemodulo == 0)
		{
			giveOrders(ann, 7);
		}
		if(!testmode && frame%30 == 0 && frame != 0 && (Broodwar->enemy()->getUnits().size() == 0 || Broodwar->self()->getUnits().size() == 0 || frame > 4000) && indcount < numIndividuals && gencount < numgenerations)
		{
			if(trialscount < trialsPerInd)
			{
				score += 1;
				score += (Broodwar->self()->getKillScore());
				
				if(Broodwar->enemy()->getUnits().size() == 0)
				{
					score += (500 - Broodwar->enemy()->getKillScore());
					score+= 500;
					printf(" v ");
				}
				trialscount++;
				Broodwar->restartGame();
			}
			else
			{
				chromosome individual = nr1.getIndividual(indcount);
				trialscount = 0;
				score /= trialsPerInd;
				
				indcount++;
				fitnesses.push_back(score);
				
				printf("\ngen: %i ind: %i score: %f id: %i gen: %i\n\n",gencount, indcount-1, score, individual.getID(), individual.gen);
				score = 0;
				shouldChange = true;
			}
		}
		else if(!testmode && frame%30 == 0 && frame != 0 && (frame > 4000 || Broodwar->enemy()->getUnits().size() == 0 || Broodwar->self()->getUnits().size())&& indcount == numIndividuals && gencount < numgenerations)
		{
			nr1.setFitness(fitnesses);
			fitnesses.clear();
			chromosome best = nr1.getBest();
			writeBestToFile(best.getGenes(), pathString, best.getFitness(), nr1.getPopulationFitness()/numIndividuals, gencount);

			printf("bestfitness: %f id: %i gen: %i\n", best.getFitness(), best.getID(), best.gen);
			gencount++;
			
			
			//printf("fr crossover");
			//nr1.newGenerationNoCross(mutationprob, mean, deviation, percentage);
			nr1.newGeneration(mutationprob, mean, deviation, percentage, gencount);
			//printf("etter crossover");
			indcount = 0;
			chromosome subject = nr1.getIndividual(0);
			ann.changeWeights(subject.getGenes());
			//shouldChange = true;
			
		}
		if(testmode && frame%30 == 0 && frame != 0 && Broodwar->enemy()->getUnits().size() == 0 && counter < numgenerations)
		{
			counter++;
			wins++;
			
			score += Broodwar->self()->getKillScore();
			printf("game nr: %i wins: %i losses: %i score: %f \n",counter, wins, losses, score/counter);
			Broodwar->restartGame();
		}
		else if(testmode && frame%30 == 0 && frame != 0 && Broodwar->self()->getUnits().size() == 0 && counter < numgenerations)
		{
			counter++;
			losses++;
			score += Broodwar->self()->getKillScore();

			printf("game nr: %i wins: %i losses: %i score: %f \n",counter, wins, losses, score/counter);
			Broodwar->restartGame();
		}
		else if(testmode && counter == numgenerations)
		{
			vector< vector<double> > v = ann.getWeights();
			stringstream s;
			s << "evobp\\" << trynumber << "evobptry" << wins << "V" << score/counter << "score.txt";
			WriteToFile(v, s.str());
			return 0;
		}



#pragma region GameEvents
      for(std::list<Event>::iterator e=Broodwar->getEvents().begin();e!=Broodwar->getEvents().end();e++)
      {
        switch(e->getType())
        {
          case EventType::MatchEnd:
            if (e->isWinner())
			{
              //printf("I won the game\n");
			}

            else
			{
				//printf("I didn't win the game\n");
			}
            break;
          case EventType::SendText:
            if (e->getText()=="/show bullets")
            {
              show_bullets=!show_bullets;
            } else if (e->getText()=="/show players")
            {
              showPlayers();
            } else if (e->getText()=="/show forces")
            {
              showForces();
            } else if (e->getText()=="/show visibility")
            {
              show_visibility_data=!show_visibility_data;
            } else if (e->getText()=="/analyze")
            {
              if (analyzed == false)
              {
                Broodwar->printf("Analyzing map... this may take a minute");
                CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)AnalyzeThread, NULL, 0, NULL);
              }
            } else
            {
              Broodwar->printf("You typed '%s'!",e->getText().c_str());
              Broodwar->sendText("%s",e->getText().c_str());
            }
            break;
          case EventType::ReceiveText:
            Broodwar->printf("%s said '%s'", e->getPlayer()->getName().c_str(), e->getText().c_str());
            break;
          case EventType::PlayerLeft:
            Broodwar->sendText("%s left the game.",e->getPlayer()->getName().c_str());
            break;
          case EventType::NukeDetect:
            if (e->getPosition()!=Positions::Unknown)
              Broodwar->printf("Nuclear Launch Detected at (%d,%d)",e->getPosition().x(),e->getPosition().y());
            else
              Broodwar->printf("Nuclear Launch Detected");
            break;
          case EventType::UnitDiscover:
            if (!Broodwar->isReplay() && Broodwar->getFrameCount()>1)
              Broodwar->sendText("A %s [%x] has been discovered at (%d,%d)",e->getUnit()->getType().getName().c_str(),e->getUnit(),e->getUnit()->getPosition().x(),e->getUnit()->getPosition().y());
            break;
          case EventType::UnitEvade:
            if (!Broodwar->isReplay() && Broodwar->getFrameCount()>1)
              Broodwar->sendText("A %s [%x] was last accessible at (%d,%d)",e->getUnit()->getType().getName().c_str(),e->getUnit(),e->getUnit()->getPosition().x(),e->getUnit()->getPosition().y());
            break;
          case EventType::UnitShow:
            if (!Broodwar->isReplay() && Broodwar->getFrameCount()>1)
              Broodwar->sendText("A %s [%x] has been spotted at (%d,%d)",e->getUnit()->getType().getName().c_str(),e->getUnit(),e->getUnit()->getPosition().x(),e->getUnit()->getPosition().y());
            break;
          case EventType::UnitHide:
            if (!Broodwar->isReplay() && Broodwar->getFrameCount()>1)
              Broodwar->sendText("A %s [%x] was last seen at (%d,%d)",e->getUnit()->getType().getName().c_str(),e->getUnit(),e->getUnit()->getPosition().x(),e->getUnit()->getPosition().y());
            break;
          case EventType::UnitCreate:
            if (Broodwar->getFrameCount()>1)
            {
              if (!Broodwar->isReplay())
                Broodwar->sendText("A %s [%x] has been created at (%d,%d)",e->getUnit()->getType().getName().c_str(),e->getUnit(),e->getUnit()->getPosition().x(),e->getUnit()->getPosition().y());
              else
              {
                /*if we are in a replay, then we will print out the build order
                (just of the buildings, not the units).*/
                if (e->getUnit()->getType().isBuilding() && e->getUnit()->getPlayer()->isNeutral()==false)
                {
                  int seconds=Broodwar->getFrameCount()/24;
                  int minutes=seconds/60;
                  seconds%=60;
                  Broodwar->sendText("%.2d:%.2d: %s creates a %s",minutes,seconds,e->getUnit()->getPlayer()->getName().c_str(),e->getUnit()->getType().getName().c_str());
                }
              }
            }
            break;
          case EventType::UnitDestroy:
            if (!Broodwar->isReplay() && Broodwar->getFrameCount()>1)
              Broodwar->sendText("A %s [%x] has been destroyed at (%d,%d)",e->getUnit()->getType().getName().c_str(),e->getUnit(),e->getUnit()->getPosition().x(),e->getUnit()->getPosition().y());
            break;
          case EventType::UnitMorph:
            if (!Broodwar->isReplay())
              Broodwar->sendText("A %s [%x] has been morphed at (%d,%d)",e->getUnit()->getType().getName().c_str(),e->getUnit(),e->getUnit()->getPosition().x(),e->getUnit()->getPosition().y());
            else
            {
              /*if we are in a replay, then we will print out the build order
              (just of the buildings, not the units).*/
              if (e->getUnit()->getType().isBuilding() && e->getUnit()->getPlayer()->isNeutral()==false)
              {
                int seconds=Broodwar->getFrameCount()/24;
                int minutes=seconds/60;
                seconds%=60;
                Broodwar->sendText("%.2d:%.2d: %s morphs a %s",minutes,seconds,e->getUnit()->getPlayer()->getName().c_str(),e->getUnit()->getType().getName().c_str());
              }
            }
            break;
          case EventType::UnitRenegade:
            if (!Broodwar->isReplay())
              Broodwar->sendText("A %s [%x] is now owned by %s",e->getUnit()->getType().getName().c_str(),e->getUnit(),e->getUnit()->getPlayer()->getName().c_str());
            break;
          case EventType::SaveGame:
            Broodwar->sendText("The game was saved to \"%s\".",e->getText().c_str());
            break;
        }
      }

      if (show_bullets)
        drawBullets();

      if (show_visibility_data)
        drawVisibilityData();

      if (!Broodwar->isReplay())
      {
        drawStats();

        if (analyzed && Broodwar->getFrameCount()%30==0)
        {
          //order one of our workers to guard our chokepoint.
          for(std::set<Unit*>::const_iterator i=Broodwar->self()->getUnits().begin();i!=Broodwar->self()->getUnits().end();i++)
          {
            if ((*i)->getType().isWorker())
            {
              //get the chokepoints linked to our home region
              std::set<BWTA::Chokepoint*> chokepoints= home->getChokepoints();
              double min_length=10000;
              BWTA::Chokepoint* choke=NULL;

              //iterate through all chokepoints and look for the one with the smallest gap (least width)
              for(std::set<BWTA::Chokepoint*>::iterator c=chokepoints.begin();c!=chokepoints.end();c++)
              {
                double length=(*c)->getWidth();
                if (length<min_length || choke==NULL)
                {
                  min_length=length;
                  choke=*c;
                }
              }

              //order the worker to move to the center of the gap
              (*i)->rightClick(choke->getCenter());
              break;
            }
          }
        }
        if (analysis_just_finished)
        {
          Broodwar->printf("Finished analyzing map.");
          analysis_just_finished=false;
        }
        if (analyzed)
          drawTerrainData();
      }

      BWAPI::BWAPIClient.update();
      if (!BWAPI::BWAPIClient.isConnected())
      {
        printf("Reconnecting...\n");
        reconnect();
      }
	  #pragma endregion
    }
    //printf("Game ended\n");

  }
  system("pause");
  return 0;

}

DWORD WINAPI AnalyzeThread()
{
  BWTA::analyze();

  //self start location only available if the map has base locations
  if (BWTA::getStartLocation(BWAPI::Broodwar->self())!=NULL)
  {
    home       = BWTA::getStartLocation(BWAPI::Broodwar->self())->getRegion();
  }
  //enemy start location only available if Complete Map Information is enabled.
  if (BWTA::getStartLocation(BWAPI::Broodwar->enemy())!=NULL)
  {
    enemy_base = BWTA::getStartLocation(BWAPI::Broodwar->enemy())->getRegion();
  }
  analyzed   = true;
  analysis_just_finished = true;
  return 0;
}

void drawStats()
{
  std::set<Unit*> myUnits = Broodwar->self()->getUnits();
  Broodwar->drawTextScreen(5,0,"I have %d units:",myUnits.size());
  std::map<UnitType, int> unitTypeCounts;
  for(std::set<Unit*>::iterator i=myUnits.begin();i!=myUnits.end();i++)
  {
    if (unitTypeCounts.find((*i)->getType())==unitTypeCounts.end())
    {
      unitTypeCounts.insert(std::make_pair((*i)->getType(),0));
    }
    unitTypeCounts.find((*i)->getType())->second++;
  }
  int line=1;
  for(std::map<UnitType,int>::iterator i=unitTypeCounts.begin();i!=unitTypeCounts.end();i++)
  {
    Broodwar->drawTextScreen(5,16*line,"- %d %ss",(*i).second, (*i).first.getName().c_str());
    line++;
  }
}

void drawBullets()
{
  std::set<Bullet*> bullets = Broodwar->getBullets();
  for(std::set<Bullet*>::iterator i=bullets.begin();i!=bullets.end();i++)
  {
    Position p=(*i)->getPosition();
    double velocityX = (*i)->getVelocityX();
    double velocityY = (*i)->getVelocityY();
    if ((*i)->getPlayer()==Broodwar->self())
    {
      Broodwar->drawLineMap(p.x(),p.y(),p.x()+(int)velocityX,p.y()+(int)velocityY,Colors::Green);
      Broodwar->drawTextMap(p.x(),p.y(),"\x07%s",(*i)->getType().getName().c_str());
    }
    else
    {
      Broodwar->drawLineMap(p.x(),p.y(),p.x()+(int)velocityX,p.y()+(int)velocityY,Colors::Red);
      Broodwar->drawTextMap(p.x(),p.y(),"\x06%s",(*i)->getType().getName().c_str());
    }
  }
}

void drawVisibilityData()
{
  for(int x=0;x<Broodwar->mapWidth();x++)
  {
    for(int y=0;y<Broodwar->mapHeight();y++)
    {
      if (Broodwar->isExplored(x,y))
      {
        if (Broodwar->isVisible(x,y))
          Broodwar->drawDotMap(x*32+16,y*32+16,Colors::Green);
        else
          Broodwar->drawDotMap(x*32+16,y*32+16,Colors::Blue);
      }
      else
        Broodwar->drawDotMap(x*32+16,y*32+16,Colors::Red);
    }
  }
}

void drawTerrainData()
{
  //we will iterate through all the base locations, and draw their outlines.
  for(std::set<BWTA::BaseLocation*>::const_iterator i=BWTA::getBaseLocations().begin();i!=BWTA::getBaseLocations().end();i++)
  {
    TilePosition p=(*i)->getTilePosition();
    Position c=(*i)->getPosition();

    //draw outline of center location
    Broodwar->drawBox(CoordinateType::Map,p.x()*32,p.y()*32,p.x()*32+4*32,p.y()*32+3*32,Colors::Blue,false);

    //draw a circle at each mineral patch
    for(std::set<BWAPI::Unit*>::const_iterator j=(*i)->getStaticMinerals().begin();j!=(*i)->getStaticMinerals().end();j++)
    {
      Position q=(*j)->getInitialPosition();
      Broodwar->drawCircle(CoordinateType::Map,q.x(),q.y(),30,Colors::Cyan,false);
    }

    //draw the outlines of vespene geysers
    for(std::set<BWAPI::Unit*>::const_iterator j=(*i)->getGeysers().begin();j!=(*i)->getGeysers().end();j++)
    {
      TilePosition q=(*j)->getInitialTilePosition();
      Broodwar->drawBox(CoordinateType::Map,q.x()*32,q.y()*32,q.x()*32+4*32,q.y()*32+2*32,Colors::Orange,false);
    }

    //if this is an island expansion, draw a yellow circle around the base location
    if ((*i)->isIsland())
      Broodwar->drawCircle(CoordinateType::Map,c.x(),c.y(),80,Colors::Yellow,false);
  }

  //we will iterate through all the regions and draw the polygon outline of it in green.
  for(std::set<BWTA::Region*>::const_iterator r=BWTA::getRegions().begin();r!=BWTA::getRegions().end();r++)
  {
    BWTA::Polygon p=(*r)->getPolygon();
    for(int j=0;j<(int)p.size();j++)
    {
      Position point1=p[j];
      Position point2=p[(j+1) % p.size()];
      Broodwar->drawLine(CoordinateType::Map,point1.x(),point1.y(),point2.x(),point2.y(),Colors::Green);
    }
  }

  //we will visualize the chokepoints with red lines
  for(std::set<BWTA::Region*>::const_iterator r=BWTA::getRegions().begin();r!=BWTA::getRegions().end();r++)
  {
    for(std::set<BWTA::Chokepoint*>::const_iterator c=(*r)->getChokepoints().begin();c!=(*r)->getChokepoints().end();c++)
    {
      Position point1=(*c)->getSides().first;
      Position point2=(*c)->getSides().second;
      Broodwar->drawLine(CoordinateType::Map,point1.x(),point1.y(),point2.x(),point2.y(),Colors::Red);
    }
  }
}

void showPlayers()
{
  std::set<Player*> players=Broodwar->getPlayers();
  for(std::set<Player*>::iterator i=players.begin();i!=players.end();i++)
  {
    Broodwar->printf("Player [%d]: %s is in force: %s",(*i)->getID(),(*i)->getName().c_str(), (*i)->getForce()->getName().c_str());
  }
}

void showForces()
{
  std::set<Force*> forces=Broodwar->getForces();
  for(std::set<Force*>::iterator i=forces.begin();i!=forces.end();i++)
  {
    std::set<Player*> players=(*i)->getPlayers();
    Broodwar->printf("Force %s has the following players:",(*i)->getName().c_str());
    for(std::set<Player*>::iterator j=players.begin();j!=players.end();j++)
    {
      Broodwar->printf("  - Player [%d]: %s",(*j)->getID(),(*j)->getName().c_str());
    }
  }
}
