#include "Population.h"
#include <time.h>
#include "chromosome.h"

Population::Population(int popSize, vector<double> seed)
{
	//srand((unsigned)time(0));
	for(int i = 0; i < popSize; i++)
	{
		chromosome ind(seed, i, 0);
		if(i != 0)
		{
			ind.mutate(0.5, 0, 1);
		}
		individuals.push_back(ind);
	}
}

Population::Population(int popSize, std::vector< vector<double> > seeds)
{
	int numseeds = seeds.size();
	for(int i = 0; i < popSize; i++)
	{
		if(i < numseeds)
		{
			chromosome chr(seeds[i], i, 0);
			individuals.push_back(chr);
		}
		else
		{
			chromosome mutate = individuals[i%numseeds];
			individuals.push_back(mutate);
		}
	}
}
Population::Population(int popSize, int genesize, double mean, double deviation)
{
	for(int i = 0; i < popSize; i++)
	{
		vector<double> tempgenes(genesize, 0);
		chromosome tempchr(tempgenes, i, 0);
		tempchr.mutate(1, mean, deviation);
		individuals.push_back(tempchr);
	}
}
Population::~Population(void)
{
}
void Population::setFitness(std::vector<double> new_fitness)
{
	for(int i = 0; i < individuals.size(); i++)
	{
		individuals[i].setFitness(new_fitness[i]);
	}
}
double Population::getPopulationFitness()
{
	double n = 0;
	for(int i = 0; i < individuals.size(); i++)
	{
		n += individuals[i].getFitness();
	}
	return n;
}
bool fitnessComparator(chromosome a, chromosome b)
{
	return a.getFitness() > b.getFitness();
}
vector<chromosome> Population::crossOver(int new_pop_size)
{
	double totalFitness = getPopulationFitness();
	double cumulativefitness = 0;
	
	vector<chromosome> tempInd = individuals;

	double random1 = rand() % (int)totalFitness;;
	

	chromosome firstParent;
	chromosome secondParent;

	for(int i = 0; i < tempInd.size(); i++)
	{
		cumulativefitness += tempInd[i].getFitness();
		if(cumulativefitness > random1)
		{
			firstParent = tempInd[i];
			tempInd.erase(tempInd.begin()+i);
		}
	}
	double random2 = rand() % (int)(totalFitness-firstParent.getFitness());
	cumulativefitness = 0;

	for(int i = 0; i < tempInd.size(); i++)
	{
		cumulativefitness += tempInd[i].getFitness();
		if(cumulativefitness > random2)
		{
			secondParent = tempInd[i];
		}
	}
	double random3 = ((double)rand()) / (RAND_MAX+1);
	int genesMax = firstParent.getGenes().size();
	random3 *= genesMax;
	int index = random3;
	
	vector<double> tempGenes1;
	vector<double> tempGenes2;
	
	for(int i = 0; i < genesMax; i++)
	{
		if(i < index)
		{
			tempGenes1.push_back(firstParent.getGenes()[i]);
			tempGenes2.push_back(secondParent.getGenes()[i]);
		}
		else
		{
			tempGenes1.push_back(secondParent.getGenes()[i]);
			tempGenes2.push_back(firstParent.getGenes()[i]);
		}
	}
	//chromosome child1(tempGenes1, new_pop_size, gencount);
	//chromosome child2(tempGenes2, new_pop_size+1, gencount);

	vector<chromosome> children;
	//children.push_back(child1);
	//children.push_back(child2);

	return children;
}
vector<chromosome> Population::crossOverESP(int new_pop_size, int hiddenlayer, int inputs, int outputs, int gencount)
{
	double totalFitness = getPopulationFitness();
	double cumulativefitness = 0;
	
	vector<chromosome> tempInd = individuals;

	double random1 = ((double)rand()) / (RAND_MAX+1);
	random1 *= totalFitness;
	
	//printf("totalf: %f random1: %f \n", totalFitness, random1);

	chromosome firstParent;
	chromosome secondParent;
	
	for(int i = 0; i < tempInd.size(); i++)
	{
		cumulativefitness += tempInd[i].getFitness();
		if(cumulativefitness > random1)
		{
			firstParent = tempInd[i];
			tempInd.erase(tempInd.begin()+i);
			break;
		}
		
	}
	//printf("firstparent: %f \n", firstParent.getFitness());

	double random2 = ((double)rand()) / (RAND_MAX+1);
	random2 *= (totalFitness-firstParent.getFitness());

	cumulativefitness = 0;

	//printf("random2: %f totalf: %f \n", random2, totalFitness);
	for(int i = 0; i < tempInd.size(); i++)
	{
		cumulativefitness += tempInd[i].getFitness();
		if(cumulativefitness > random2)
		{
			secondParent = tempInd[i];
			break;
		}
	}

	//printf("second: %f \n", secondParent.getFitness());

	double random3 = ((double)rand()) / (RAND_MAX+1);
	int genesMax = firstParent.getGenes().size();
	random3 *= genesMax;
	int index = random3;
	
	vector<double> tempGenes1;
	vector<double> tempGenes2;

	int numCrossoverNeuron = (rand() % (hiddenlayer-1));
	//printf("%i p1: %i p2: %i", numCrossoverNeuron, firstParent.getGenes().size(), secondParent.getGenes().size());

	for(int i = 0; i < hiddenlayer*inputs; i++)
	{
		if(i%hiddenlayer <= numCrossoverNeuron)
		{
			tempGenes1.push_back(firstParent.getGenes()[i]);
			tempGenes2.push_back(secondParent.getGenes()[i]);
		}
		else
		{
			tempGenes1.push_back(secondParent.getGenes()[i]);
			tempGenes2.push_back(firstParent.getGenes()[i]);
		}
	}
	for(int i = hiddenlayer*inputs; i < genesMax; i++)
	{
		if((i-hiddenlayer*inputs) < numCrossoverNeuron*inputs)
		{
			tempGenes1.push_back(firstParent.getGenes()[i]);
			tempGenes2.push_back(secondParent.getGenes()[i]);
		}
		else
		{
			tempGenes1.push_back(secondParent.getGenes()[i]);
			tempGenes2.push_back(firstParent.getGenes()[i]);
		}
	}
	
	chromosome child1(tempGenes1, new_pop_size, gencount);
	chromosome child2(tempGenes2, new_pop_size+1, gencount);

	vector<chromosome> children;
	children.push_back(child1);
	children.push_back(child2);

	return children;
}
void Population::newGenerationNoCross(double mutprob, double mean, double deviation, double percentage)
{
	vector<chromosome> new_generation;
	list<chromosome> chrList(individuals.begin(), individuals.end());
	chrList.sort(fitnessComparator);
	

	int elite = individuals.size() * percentage;
	int elitecount = 0;

	for(list<chromosome>::iterator i = chrList.begin(); i != chrList.end(); i++)
	{
		if(elitecount < elite)
		{
			new_generation.push_back((*i));
			elitecount++;
		}
	}

	for(int i = 0; i < individuals.size()-elite; i++)
	{
		chromosome chr = new_generation[i%elite];
		chr.mutate(mutprob, mean, deviation);
		new_generation.push_back(chr);
	}
	while(new_generation.size() > individuals.size())
	{
		new_generation.pop_back();
	}
	individuals.clear();

	for(int i = 0; i < new_generation.size(); i++)
	{
		
		individuals.push_back(new_generation[i]);
		individuals[i].setFitness(0.0);
	}
	new_generation.clear();

}
void Population::newGeneration(double mutprob, double mean, double deviation, double elitism, int gencount)
{
	vector<chromosome> new_generation;

	list<chromosome> chlist(individuals.begin(), individuals.end());
	chlist.sort(fitnessComparator);

	elitism *= individuals.size();
	for(int i = 0; i < elitism; i++)
	{
		chromosome best = chlist.front();
		chlist.pop_front();
		new_generation.push_back(best);
	}
	int maxindex = individuals.size()/2;
	
	for(int i = 0; i < maxindex; i++)
	{
		vector<chromosome> tempNew = crossOverESP(new_generation.size(), 5, 29, 6, gencount);
		
		chromosome temp1 = tempNew[0];
		chromosome temp2 = tempNew[1];
		
		temp1.mutate(mutprob, mean, deviation);
		temp2.mutate(mutprob, mean, deviation);

		new_generation.push_back(temp1);
		new_generation.push_back(temp2);
	}
	while(new_generation.size() > individuals.size())
	{
		new_generation.pop_back();
	}

	individuals.clear();

	for(int i = 0; i < new_generation.size(); i++)
	{
		
		individuals.push_back(new_generation[i]);
		individuals[i].setFitness(0.0);
	}

	new_generation.clear();
}
chromosome Population::getIndividual(int index)
{
	return individuals[index];
}


chromosome Population::getBest()
{
	chromosome best = individuals[0];
	for(int i = 1; i < individuals.size(); i++)
	{
		if(best.getFitness() < individuals[i].getFitness())
		{
			best = individuals[i];
		}
	}
	return best;
}